@@ -39,6 +39,7 @@
 #include <X11/Xutil.h>
 
 #include "mozplugger.h"
+#include "mozplugger-child.h"
 
 #define MAX_CONTROLS_WIDTH           300
 #define MAX_CONTROLS_HEIGHT          100
@@ -71,10 +72,6 @@ static int repeatsResetVal;
 
 static int pipe_fd;
 static int flags;
-static int sig_fds[2];
-
-#define SIG_RD_FD 0
-#define SIG_WR_FD 1
 
 static GC gc_white;
 static GC gc_black;
@@ -320,6 +317,8 @@ static int igetenv(char *var, int def)
  *****************************************************************************/
 static void my_play(char * command)
 {
+     int maxFd;
+
      if(state != STATE_STOP)
      {
           if(!kill(-pid, SIGCONT))
@@ -329,63 +328,27 @@ static void my_play(char * command)
           }
      }
 
-     pid=fork();
+     maxFd = pipe_fd;
+     if(dpy && ConnectionNumber(dpy) > maxFd)
+     {
+         maxFd = ConnectionNumber(dpy);
+     }
+
+     pid = spawn_app(command, flags, maxFd);
      if(pid == -1)
      {
           state=STATE_STOP;
           return;
      }
-     if(pid == 0)
+
+     state=STATE_PLAY;
+     if(!repeats) 
      {
-          char *cmd[4];
-          /* Child process, so as usually need to cleanup any inherited
-           * file descriptors */
-          if(dpy)
-          {
-               close(ConnectionNumber(dpy));
-          }
-          if(sig_fds[SIG_RD_FD] >= 0)
-          {
-               close(sig_fds[SIG_RD_FD]);
-          }
-          if(sig_fds[SIG_WR_FD] >= 0)
-          {
-               close(sig_fds[SIG_WR_FD]);
-          }
-          close(pipe_fd);
-
-          /* Redirect stdout & stderr to /dev/null */
-          if(flags & H_NOISY)
-          {
-               const int ofd = open("/dev/null", O_RDONLY);
-               if(ofd)
-               {
-                    dup2(ofd, 1);
-                    dup2(ofd, 2);
-                    close(ofd);
-               }
-          }
-
-          setpgid(pid, 0);
-
-          cmd[0]="/bin/sh";
-          cmd[1]="-c";
-          cmd[2]=command;
-          cmd[3]=0;
-          execvp(cmd[0], cmd);
-          exit(EX_UNAVAILABLE);
+          repeats = repeatsResetVal;
      }
-     else
+     if((repeats > 0) && (repeats != MAXINT)) 
      {
-          state=STATE_PLAY;
-          if(!repeats) 
-          {
-               repeats = repeatsResetVal;
-          }
-          if((repeats > 0) && (repeats != MAXINT)) 
-          {
-               repeats--;
-          }
+          repeats--;
      }
 }
 
@@ -508,26 +471,6 @@ static void sigdie(int sig)
      low_die(); 
 }
 
-/******************************************************************************/
-/**
- * @brief Convert signal to message in pipe
- *
- * Callback function attached as a signal handler. This sends a message over
- * a pipe about the signal that has just happened. This allows signals to
- * be handled by select() 
- *
- * @param[in] sig The signal
- *
- *****************************************************************************/
-static void sig_to_fd(int sig)
-{
-     D("Signal %i routed to pipe\n", sig);
-     if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
-     {
-          D("Signal write to pipe failed\n");
-     }
-}
-
 /******************************************************************************/
 /**
  * Checks for X events and processes them. Returns when no more events left
@@ -891,22 +834,12 @@ int main(int argc, char **argv)
 
      old_state=state;
 
-     if(pipe(sig_fds) != 0)
-     {
-          D("Failed to create pipe to handle signals");
-          sig_fds[SIG_RD_FD] = sig_fds[SIG_WR_FD] = -1;
-     }
+     redirect_SIGCHLD_to_fd();
 
      signal(SIGHUP, sigdie);
      signal(SIGINT, sigdie);
      signal(SIGTERM, sigdie);
 
-     if(sig_fds[SIG_WR_FD] >= 0)
-     {
-          /* Re-direct SIGCHLD events through a pipe to be see by select() */
-          signal(SIGCHLD, sig_to_fd);
-     }
-
      if(igetenv("autostart",1))
      {
           my_play(command);
@@ -916,6 +849,7 @@ int main(int argc, char **argv)
      {
           fd_set fds;
           int maxFd;
+          int sig_chld_fd;
 
           if(state != old_state)
           {
@@ -932,10 +866,11 @@ int main(int argc, char **argv)
 
           maxFd = MAX(ConnectionNumber(dpy), pipe_fd);
 
-          if(sig_fds[SIG_RD_FD] >= 0)
+          sig_chld_fd = get_SIGCHLD_fd();
+          if(sig_chld_fd >= 0)
           {
-               FD_SET(sig_fds[SIG_RD_FD], &fds);
-               maxFd = MAX(maxFd, sig_fds[SIG_RD_FD]);
+               FD_SET(sig_chld_fd, &fds);
+               maxFd = MAX(maxFd, sig_chld_fd);
           }
 
           D("SELECT IN maxFd = %i\n", maxFd);
@@ -945,10 +880,9 @@ int main(int argc, char **argv)
                {    
                     check_pipe_fd_events();
                }
-               if(FD_ISSET(sig_fds[SIG_RD_FD], &fds))
+               if(FD_ISSET(sig_chld_fd, &fds))
                {
-                    int sig;
-                    read(sig_fds[SIG_RD_FD], &sig, sizeof(int));
+                    handle_SIGCHLD_event();
                }
           }
           D("SELECT OUT\n");
