@@ -43,6 +43,7 @@
 #include <sys/time.h>
 
 #include "mozplugger.h"
+#include "mozplugger-child.h"
 
 /*****************************************************************************
  * Type declarations
@@ -80,10 +81,6 @@ static int repeats;
 static char *winname;
 static char *command;
 static char *file;
-static int sig_fds[2];
-
-#define SIG_RD_FD  0
-#define SIG_WR_FD  1
 
 static XWindowAttributes wattr;
 
@@ -136,7 +133,7 @@ static int error_handler(Display *dpy, XErrorEvent *err)
 static void initSwallowMutex(void)
 {
      D("Initialising Swallow Mutex\n"); 
-     if(!display) 
+     if(display == 0) 
      {
 	  fprintf(stderr, "Display not set so cannot initialise semaphore!\n");
      }
@@ -806,7 +803,7 @@ static int setup_display(void)
      if(parentDetails.window == 0)       /* mozdev bug #18837 */
      {
           D("setup_display() WINDOW is Null - so nothing setup\n");
-          return 1;
+          return 0;
      }
 
      displayname = getenv("DISPLAY");
@@ -814,10 +811,11 @@ static int setup_display(void)
 
      XSetErrorHandler(error_handler);
 
-     if (!(display = XOpenDisplay(displayname)))
+     display = XOpenDisplay(displayname);
+     if(display == 0)
      {
           D("setup_display() failed cannot open display!!\n");
-	  return 1;
+	  return 0;
      }
 
      if (!XGetWindowAttributes(display, parentDetails.window, &wattr))
@@ -825,7 +823,7 @@ static int setup_display(void)
           D("setup_display() failed cannot get window attributes!!\n");
           XCloseDisplay(display);
           display = 0;
-          return 1;
+          return 0;
      }
      D("display=0x%x\n", (unsigned) display);
      D("WINDOW =0x%x\n", (unsigned) parentDetails.window);
@@ -833,81 +831,7 @@ static int setup_display(void)
      
      D("setup_display() done\n");
 
-     return 0;
-}
-
-/*****************************************************************************/
-/** 
- * Wrapper for execlp() that calls the application.
- *
- * WARNING: This function runs in the daughter process so one must assume the
- * daughter uses a copy (including) heap memory of the parent's memory space
- * i.e. any write to memory here does not affect the parent memory space. 
- * Since Linux uses copy-on-write, best leave memory read-only and once execlp
- * is called, all daughter memory is wiped anyway (except the stack).
- *
- * @param[in] argv Pointer to list of arguments
- *
- * @return none
- *
- *****************************************************************************/
-static void run_app(char **argv)
-{
-     /* Close all File descriptors inherited from the parent,
-      * that's the XServer connection FD and pipe_fd */
-     if(display)
-     {
-          close(ConnectionNumber(display));
-     }
-     if(sig_fds[SIG_RD_FD] >= 0)
-     {
-          close(sig_fds[SIG_RD_FD]);
-     }
-     if(sig_fds[SIG_WR_FD] >= 0)
-     {
-          close(sig_fds[SIG_WR_FD]);
-     }
-     close(pipe_fd);
-
-     /* Redirect stdout & stderr to /dev/null */
-     if(flags & (H_NOISY | H_DAEMON))
-     {
-	  const int ofd = open("/dev/null", O_RDONLY);
-
-	  D("Redirecting stdout and stderr\n");
-
-	  if(ofd == -1)
-          {
-	       exit(EX_UNAVAILABLE);
-          }
-
-	  dup2(ofd, 1);
-	  dup2(ofd, 2);
-	  close(ofd);
-     }
-
-     /* For a DAEMON we must also redirect STDIN to /dev/null
-      * otherwise we can get some weird behaviour especially with realplayer
-      * (I suspect it uses stdin to communicate with its peers) */
-     if(flags & H_DAEMON)
-     {
-          const int ifd = open("/dev/null", O_WRONLY);
-
-          D("Redirecting stdin also\n");
-
-          if(ifd == -1)
-          {
-               exit(EX_UNAVAILABLE);
-          }
-          dup2(ifd, 0);
-          close(ifd);
-     }
-#if DEBUG
-     close_debug();
-#endif
-     execvp(argv[0], argv);
-     D("Execvp failed. (errno=%d)\n", errno);
-     exit(EX_UNAVAILABLE);
+     return 1;
 }
 
 /*****************************************************************************/
@@ -929,33 +853,6 @@ static void add_possible_victim(Window window)
      }
 }
 
-/*****************************************************************************/
-/**
- * Test if the window provided is a possible victim window, this uses the
- * list of possible windows.
- *
- * @param[in] window  The window to be tested
- * @param[in] name    The name of the window being looked for
- *
- * @return true if yes
- *
- *****************************************************************************/
-static int test_possible_victim(Window window, const char *name)
-{
-     D("test_possible_victim, testing window 0x%x\n",(unsigned)window);
-     
-     if (check_window_name(window, name))
-     {
-          /* See if some instance of mozplugger got the window before us, if
-           * not mark it as ours */
-          if(chkAndMarkVictimWindow(window))
-          {
-               return true;
-          }
-     }
-     return false;
-}
-
 /*****************************************************************************/
 /**
  * Checks if the passed window is the victim.
@@ -969,6 +866,8 @@ static int find_victim(Window window)
 {
      if (!victimDetails.window)
      {
+          Window found = 0;
+
 	  D("Looking for victim... (%s)\n", winname);
 
           /* New way, check through list of newly created windows */
@@ -977,26 +876,31 @@ static int find_victim(Window window)
           {
                if(window == possible_victim_windows[i])
 	       {
-                    if(test_possible_victim(window, winname))
+                    if (check_window_name(window, winname))
                     {
-                         victimDetails.window = window;
+                         found = true;
                          break;
                     }
                }
           }
 
-	  if (victimDetails.window)
+	  if (found)
 	  {
 	       XWindowAttributes ca;
-	       if(XGetWindowAttributes(display, victimDetails.window, &ca))
+	       if(XGetWindowAttributes(display, window, &ca))
                {
-                    victimDetails.borderWidth = ca.border_width;
-                    victimDetails.x = ca.x;
-                    victimDetails.y = ca.y;
-                    victimDetails.width = ca.width;
-                    victimDetails.height = ca.height;
+                    /* See if some instance of mozplugger got the window 
+                     * before us, if not mark it as ours */
+                    if(chkAndMarkVictimWindow(window))
+                    {
+                         victimDetails.window = window;
+                         victimDetails.borderWidth = ca.border_width;
+                         victimDetails.x = ca.x;
+                         victimDetails.y = ca.y;
+                         victimDetails.width = ca.width;
+                         victimDetails.height = ca.height;
 
-   	            D("Found victim=0x%x, x=%i, y=%i, width=%i, height=%i, "
+      	                 D("Found victim=0x%x, x=%i, y=%i, width=%i, height=%i, "
                                                                   "border=%i\n",
                                           (unsigned) victimDetails.window, 
                                                      victimDetails.x,
@@ -1005,28 +909,34 @@ static int find_victim(Window window)
                                                      victimDetails.height,
                                                      victimDetails.borderWidth);
 
-                    /* To avoid losing events, enable monitoring events on victim 
-                     * at earlist opportunity */
+                         /* To avoid losing events, enable monitoring events on 
+                          * victim at earlist opportunity */
 
-                    XSelectInput(display, victimDetails.window, StructureNotifyMask);
-                    XSync(display, False);
+                         XSelectInput(display, victimDetails.window, 
+                                                           StructureNotifyMask);
+                         XSync(display, False);
 
-                    XSelectInput(display, wattr.root, 0);
+                         XSelectInput(display, wattr.root, 0);
 
 #ifdef USE_MUTEX_LOCK
-	            giveSwallowMutex();
+	                 giveSwallowMutex();
 #endif 	
-	            if (flags & H_MAXASPECT)
-	            {
-		         set_aspect(ca.width, ca.height);
-	            }
-                    return true;
+          	         if (flags & H_MAXASPECT)
+	                 {
+	        	      set_aspect(ca.width, ca.height);
+	                 }
+                         return true;
+                    }
+                    else
+                    {
+                         D("Window 0x%x already claimed by another"
+                                " instance of mozplugger\n", (unsigned) window);
+                    }
                }
                else
                {    
                     D("XGetWindowAttributes failed for 0x%x\n", 
-                                               (unsigned) victimDetails.window);
-                    victimDetails.window = 0;
+                                                             (unsigned) window);
                }
 	  }
      }
@@ -1071,22 +981,6 @@ static void handle_rootWindow_event(const XEvent * const ev)
           /* All toplevel new app windows will be created on desktop (root),
            * so lets keep a list as they are created */
           add_possible_victim(ev->xcreatewindow.window);
-
-	  /* A window has been mapped, if window manager is allowed to
-           * fiddle, lets see if this is the window */
-          if(!ev->xcreatewindow.override_redirect)
-          {
-               if ((flags & H_HIDDEN) != 0)
-               {
-                    if(find_victim(ev->xcreatewindow.window))
-                    {
-                         /* Only happens if we have the H_HIDDEN flag as this 
-                          * means we can grab the window before the MapRequest
-                          * has happened! */
-                         reparent_window();
-                    }
-               }
-          }
 	  break;
 
      case MapNotify:
@@ -1097,7 +991,8 @@ static void handle_rootWindow_event(const XEvent * const ev)
            * fiddle, lets see if this is the window */
           if(!ev->xmap.override_redirect)
           {
-               if(find_victim(ev->xmap.window))
+               Window wnd = ev->xmap.window;
+               if(find_victim(wnd))
                {
                     /* If we get MapNotify on root before ReparentNotify for our
                      * victim this means either:-
@@ -1127,28 +1022,26 @@ static void handle_rootWindow_event(const XEvent * const ev)
            * fiddle, lets see if this is the window  we are looking for */
           if(!ev->xreparent.override_redirect)
           {
-               if(find_victim(ev->xreparent.window))
+               Window wnd = ev->xreparent.window;
+
+               if(find_victim(wnd))
                {
-                   /* If we get ReparentNotify on root before MapNotify this 
-                    * means the MapRequest has occurred and been intercepted by
-                    * the  window manager and turned into a reparenting the 
-                    * window for window decorations. It is safe to override this
-                    * with another reparenting. */
+                    /* Avoid the fight with window manager to get reparent,
+                     * instead be kind and withdraw window and wait for WM
+                     * to reparent window back to root */
+                   D("Withdraw window 0x%x\n", (unsigned) wnd);
+
+                   XWithdrawWindow(display, wnd, DefaultScreen(display));
+#if 0
 		   change_leader();
                    reparent_window();
+#endif
                }
           }
 	  break;
 
 
 #ifdef DEBUG
-     case UnmapNotify:
-	  D("***UnmapNotify for root, window=0x%x, from_configure=%i, "
-                                "send_event=%i\n", (unsigned) ev->xunmap.window,
-                                                      ev->xunmap.from_configure,
-                                                         ev->xunmap.send_event);
-          break;
-
      case ConfigureNotify:
           D("***ConfigureNotify for root, window=0x%x, "
                       "x=%i, y=%i, width=%i, height=%i, override_redirect=%i\n",
@@ -1157,8 +1050,18 @@ static void handle_rootWindow_event(const XEvent * const ev)
                                               ev->xconfigure.width,
                                               ev->xconfigure.height,
 					      ev->xconfigure.override_redirect);	
-	  break;
- 
+          break;
+
+
+
+     case UnmapNotify:
+	  D("***UnmapNotify for root, window=0x%x, from_configure=%i, "
+                                "send_event=%i\n", (unsigned) ev->xunmap.window,
+                                                      ev->xunmap.from_configure,
+                                                         ev->xunmap.send_event);
+          break;
+
+
      case DestroyNotify:
           D("***DestroyNotify for root, window=0x%x\n", 
                                           (unsigned) ev->xdestroywindow.window);
@@ -1491,6 +1394,8 @@ static void check_all_events(pid_t pid)
      while(1)
      {
 	  int selectRetVal;
+          int sig_chld_fd;
+
 	  struct timeval   timeout;
           struct timeval * pTimeout = 0;
           if (display)
@@ -1504,10 +1409,11 @@ static void check_all_events(pid_t pid)
 
           maxfd = MAX(ConnectionNumber(display), pipe_fd);
 
-          if(sig_fds[SIG_RD_FD] >= 0)
+          sig_chld_fd = get_SIGCHLD_fd();
+          if(sig_chld_fd >= 0)
           {
-               FD_SET(sig_fds[SIG_RD_FD], &fds);
-               maxfd = MAX(maxfd, sig_fds[SIG_RD_FD]);
+               FD_SET(sig_chld_fd, &fds);
+               maxfd = MAX(maxfd, sig_chld_fd);
           }
 
           /* If NX is is use and nxagent is configured as rootless, it can
@@ -1532,10 +1438,9 @@ static void check_all_events(pid_t pid)
                {
 	            check_pipe_fd_events();
                }
-               if( FD_ISSET(sig_fds[SIG_RD_FD], &fds))
+               if( FD_ISSET(sig_chld_fd, &fds))
                {
-                    int sig;
-                    read(sig_fds[SIG_RD_FD], &sig, sizeof(sig));
+                    handle_SIGCHLD_event();
                }  
           }
 	  else if((selectRetVal == 0) && pTimeout)
@@ -1556,35 +1461,18 @@ static void check_all_events(pid_t pid)
                     pid = -1;     
                     if((flags & H_DAEMON) == 0)
                     {
+                         D("Child process has died status=%i\n", status);
                          return;
                     }
-                    D("check_all_events: child process has died, keep going\n");
+                    else
+                    {
+                        D("Child process has detached, keep going\n");
+                    }
                }
           }
      }
 }
 
-/******************************************************************************/
-/**
- * Convert signal to message in pipe
- * Callback function attached as a signal handler. This sends a message over
- * a pipe about the signal that has just happened. This allows signals to
- * be handled by select() 
- *
- * @param[in] sig The signal
- *
- * @return none
- *
- *****************************************************************************/
-static void sig_to_fd(int sig)
-{
-     D("Signal %i routed to pipe\n", sig);
-     if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
-     {
-          D("Signal write to pipe failed\n");
-     }
-}
-
 /******************************************************************************/
 /**
  * Handle the application
@@ -1812,24 +1700,21 @@ int main(int argc, char **argv)
        file,
        command);
 
-     /* Create pipe to handle the signal handler to select interface */
-     if(pipe(sig_fds) != 0)
-     {
-          D("Failed to create pipe to handle signals");
-          sig_fds[SIG_RD_FD] = sig_fds[SIG_WR_FD] = -1;
-     }
-
-
-     if(sig_fds[SIG_WR_FD] >= 0)
-     {
-          /* Re-direct SIGCHLD events through a pipe to be see by select() */
-          signal(SIGCHLD, sig_to_fd);
-     }
+     redirect_SIGCHLD_to_fd();
 
      /* Create handler for when terminating the helper */
 
      signal(SIGTERM, sigTerm);
 
+     if (!setup_display())
+     {
+         if( (flags & H_SWALLOW) != 0)
+         { 
+	      D("Failed to open X display - canceling swallow functionality\n");
+	      flags &=~ H_SWALLOW;
+         }
+     }
+
      if (repeats < 1) 
      {
           repeats = 1;
@@ -1837,9 +1722,9 @@ int main(int argc, char **argv)
 
      while (repeats > 0)
      {
-	  char *app_argv[10];
 	  int loops = 1;
 	  pid_t pid;
+          int maxFd;
 
 	  /* This application will use the $repeat variable */
 	  if (flags & H_REPEATCOUNT)
@@ -1853,22 +1738,6 @@ int main(int argc, char **argv)
 	       loops = MAXINT;
           }
 
-	  /* Create command line */
-	  app_argv[0] = "/bin/sh";
-	  app_argv[1] = "-c";
-	  app_argv[2] = command;
-	  app_argv[3] = NULL;
-	  
-	  if (setup_display())
-	  {
-               if(flags & H_SWALLOW)
-               { 
-	            D("Canceling swallow functionality\n");
-	            flags &=~ H_SWALLOW;
-               }
-	  }
-
-
 	  if (flags & H_SWALLOW)
 	  {
                initWindowOwnerMarker();
@@ -1898,7 +1767,14 @@ int main(int argc, char **argv)
 	       XSync(display, False);
 	  }
 
-	  if ((pid = fork()) == -1)
+          maxFd = pipe_fd;
+          if(display && (ConnectionNumber(display) > maxFd))
+          {
+               maxFd = ConnectionNumber(display);
+          }
+
+          pid = spawn_app(command, flags, maxFd);
+	  if(pid == -1)
           {
 #ifdef USE_MUTEX_LOCK
                giveSwallowMutex();  
@@ -1906,29 +1782,21 @@ int main(int argc, char **argv)
 	       exit(EX_UNAVAILABLE);
           }
      
-	  if (pid == 0)
-	  {
-               /* Child process */
-	       D("Running %s\n", command);
-	       run_app(app_argv);
+	  D("Waiting for pid=%d\n", pid);
+
+	  handle_app(pid);
+
+	  D("Wait done (repeats=%d, loops=%d)\n", repeats, loops);
+	  if (repeats < MAXINT)
+          {
+	       repeats -= loops;
 	  }
-	  else
-	  {
+    }
 
-	       D("Waiting for pid=%d\n", pid);
-
-	       handle_app(pid);
-
-	       D("Wait done (repeats=%d, loops=%d)\n", repeats, loops);
-	       if (repeats < MAXINT)
-               {
-		    repeats -= loops;
-               }
-	  }
-          /* On each repeat we re-open the XServer connection, so need to
-           * close it at end of loop - TODO perhaps we should keep in open
-           * for all repeats? */
+    if(display)
+    {
           XCloseDisplay(display);
+          display = 0;
      }
 
      exit(0);
