@@ -127,8 +127,6 @@ static int  staticPoolIdx = 0;
 
 /*****************************************************************************/
 /**
- * @brief Wrapper for fork
- *
  * Wrapper for fork() which handles some magic needed to prevent netscape 
  * from freaking out. It also prevents interference from signals.
  * Also it closes all those file descriptors inherited from the parent 
@@ -224,16 +222,16 @@ static pid_t my_fork(data_t * const THIS, int pipeFd)
 
 /*****************************************************************************/
 /**
- * @brief Wrapper for putenv().
- *
- * Instead of writing to the envirnoment, the envirnoment variables
- * are written to a buffer.
+ * Wrapper for putenv(). Instead of writing to the envirnoment, the envirnoment
+ * variables are written to a buffer.
  *
  * @param[in,out] buffer The buffer where the envirnoment variables are written
  * @param[in,out] offset The current position in the buffer
  * @param[in] var        The name of the environment variable
  * @param[in] value      The value of the environment variable
  *
+ * @return none
+ *
  *****************************************************************************/
 static void my_putenv(char *buffer, int *offset, const char *var, 
                                                  const char *value)
@@ -252,7 +250,7 @@ static void my_putenv(char *buffer, int *offset, const char *var,
 
 /*****************************************************************************/
 /**
- * @brief Wrapper for execlp() that calls the helper.
+ * Wrapper for execlp() that calls the helper.
  *
  * WARNING: This function runs in the daughter process so one must assume the
  * daughter uses a copy (including) heap memory of the parent's memory space
@@ -265,6 +263,8 @@ static void my_putenv(char *buffer, int *offset, const char *var,
  * @param[in] file     The url of the embedded object
  * @param[in] fd       The file descriptor of the pipe to the helper application
  *
+ * @return none
+ *
  *****************************************************************************/
 static void run(data_t * const THIS, const char *file, int fd)
 {
@@ -403,9 +403,8 @@ static void run(data_t * const THIS, const char *file, int fd)
 
 /*****************************************************************************/
 /**
- * @brief Check if 'file' is somewhere to be found in the user PATH.
- *
- * Performs a stat of the path/file to see if the file exists
+ * Check if 'file' is somewhere to be found in the user PATH.  Performs a stat
+ * of the path/file to see if the file exists
  *
  * @param[in] path The path part of the whole path/file name
  * @param[in] file The file part of the whole path/file name
@@ -446,8 +445,6 @@ static int inpath(char *path, const char *file)
 
 /*****************************************************************************/
 /**
- * @brief Find a file
- *
  * Check if 'file' exists. Uses a cache of previous finds to avoid performing
  * a stat call on the same file over and over again.
  *
@@ -522,8 +519,6 @@ static int find(const char *file)
 
 /*****************************************************************************/
 /**
- * @brief Allocate some memory
- *
  * Allocate some memory from the static pool. We use a static pool for
  * the database because Mozilla can unload the plugin after use and this
  * would lead to memory leaks if we use the heap.
@@ -555,8 +550,7 @@ static void * allocStaticMem(int size)
 
 /*****************************************************************************/
 /**
- * @brief Make a dynamic string static by copying to static memory
- *
+ * Make a dynamic string static by copying to static memory.
  * Given a pointer to a string in temporary memory, return the same string
  * but this time stored in permanent (i.e. static) memory. Will only be deleted
  * when the plugin is unloaded by Mozilla.
@@ -587,9 +581,8 @@ static const char * makeStrStatic(const char * str, int len)
 
 /*****************************************************************************/
 /**
- * @brief get parameter from mozpluggerrc
- *
- * Match a word to a flag that takes a parameter e.g. swallow(name).
+ * get parameter from mozpluggerrc.  Match a word to a flag that takes a
+ * parameter e.g. swallow(name).
  *
  * @param[in] x    Pointer to position in text being parsed
  * @param[in] flag Pointer to flag being checked for
@@ -639,8 +632,6 @@ static char *get_parameter(char *x, const char *flag, const char **c)
 
 /*****************************************************************************/
 /**
- * @brief Check if line starts with the keyword
- *
  * Checks if 'line' starts with 'kw' (keyword), if 'line' is longer than 'kw', 
  * checks that there is not an additional alpha-numeric letter in the 'line'
  * after the 'kw'.
@@ -660,9 +651,8 @@ static int match_word(const char *line, const char *kw)
 }
 
 /*****************************************************************************/
-/** @brief Parse for flags
- *
- * Scan a line for all the possible flags.
+/** 
+ * Parse for flags. Scan a line for all the possible flags.
  *
  * @param [in,out] x       The position in the line that is being parsed
  *                         (before & after)
@@ -734,8 +724,6 @@ static int parse_flags(char **x, command_t *commandp)
 
 /*****************************************************************************/
 /**
- * @brief Read config file
- *
  * Read the configuration file into memory.
  *
  * @param[in] f The FILE pointer
@@ -978,8 +966,7 @@ static void read_config(FILE *f)
 }
 
 /*****************************************************************************/
-/** @brief Find location of file/app
- *
+/**
  * Find the config file or the helper file in function of the function cb.
  *
  * @param[in] basename File or app name (without path)
@@ -1068,10 +1055,8 @@ static int find_helper_file(const char *basename, int (*cb)(const char *))
 
 /*****************************************************************************/
 /**
- * @brief Read config callback function
- *
- * Open and run the configuration file (fname) through M4 and then call 
- * read_config() to parse the output
+ * Read config callback function. Open and run the configuration file (fname)
+ * through M4 and then call read_config() to parse the output
  *
  * @param[in] fname Full name (and path) of configuration file
  *
@@ -1155,8 +1140,7 @@ static int read_config_cb(const char *fname)
 
 /*****************************************************************************/
 /**
- * @brief Find mozplugger helper app callback function
- *
+ * Find mozplugger helper app callback function.
  * Check if the helper executable exists (using stat), if so save the filename
  * to the global 'helper_fname' variable
  *
@@ -1179,8 +1163,7 @@ static int find_plugger_helper_cb(const char *fname)
 
 /*****************************************************************************/
 /**
- * @brief Find mozplugger helper app callback function
- *
+ * Find mozplugger helper app callback function.
  * Check if the helper executable exists (using stat), if so save the filename
  * to the global 'controller_fname' variable
  *
@@ -1203,8 +1186,7 @@ static int find_plugger_controller_cb(const char *fname)
 
 /*****************************************************************************/
 /**
- * @brief Find mozplugger helper app callback function
- *
+ * Find mozplugger helper app callback function.
  * Check if the helper executable exists (using stat), if so save the filename
  * to the global 'linker_fname' variable
  *
@@ -1228,12 +1210,12 @@ static int find_plugger_linker_cb(const char *fname)
 
 /*****************************************************************************/
 /** 
- * @brief Read in configuration, etc
- *
  * Find configuration file, helper and controller executables. Call the
  * appropriate xxx_cb function to handle the action (e.g. for configuration
  * file, the parsering and reading into memory).
  *
+ * @return none
+ *
  *****************************************************************************/
 static void do_read_config(void)
 {
@@ -1302,10 +1284,8 @@ static void do_read_config(void)
 
 /*****************************************************************************/
 /**
- * @brief Check URL is safe
- *
- * Since href's are passed to an app as an argument, just check for ways that 
- * a shell can be tricked into executing a command.
+ * Check URL is safe. Since href's are passed to an app as an argument, just 
+ * check for ways that  a shell can be tricked into executing a command.
  *
  * @param[in] url The URL to check
  *
@@ -1337,8 +1317,6 @@ static int safeURL(const char* url)
 
 /*****************************************************************************/
 /**
- * @brief Extract fragment from URL
- *
  * Extract the 'fragment' from the end of the URL if present. This is passed
  * via the fragment environment varaible to the helper application
  *
@@ -1365,8 +1343,6 @@ static void parseURL(data_t * const THIS, const char * url)
 	
 /*****************************************************************************/
 /**
- * @brief Match URL
- *
  * See if the URL matches out match criteria.
  *
  * @param[in] matchStr The string to match
@@ -1418,8 +1394,6 @@ static int match_url(const char * matchStr, const char * url)
 
 /*****************************************************************************/
 /**
- * @brief Match command
- *
  * Go through the commands in the config file and find one that fits our needs.
  *
  * @param[in] THIS       Pointer to the data associated with this instance of the
@@ -1477,8 +1451,6 @@ static int match_command(data_t * const THIS, int streamOnly,
 
 /*****************************************************************************/
 /**
- * @brief Match mime types
- *
  * See if mimetype matches.
  *
  * @param[in] THIS     pointer to data associated with this instance of the 
@@ -1495,22 +1467,23 @@ static int match_mime_type(data_t * const THIS, mimetype_t *m)
      sscanf(m->type, "%"SMALL_BUFFER_SIZE_STR"[^:]", mimetype);
      sscanf(mimetype, "%s", mimetype);
 
-     D("Checking '%s' ?= '%s'\n", mimetype, THIS->mimetype);
+     int retVal;
      if ((strcasecmp(mimetype, THIS->mimetype) != 0) 
-             && (strcmp(mimetype,"*") != 0))
+                                                 && (strcmp(mimetype,"*") != 0))
      {
-          D("Not Same\n");
-	  return 0;
+          retVal = 0;
      }
-
-     D("Same.\n");
-     return 1;
+     else
+     {
+          retVal = 1;
+     }
+     D("Checking '%s' ?= '%s', %s\n", mimetype, THIS->mimetype, 
+                                            retVal == 1 ? "same" : "different");
+     return retVal;
 }
 
 /*****************************************************************************/
 /**
- * @brief Match handler
- *
  * See if handler matches.
  *
  * @param[in] h          Pointer to handler to match against
@@ -1556,8 +1529,6 @@ static int match_handler(handler_t *h, data_t * const THIS,
 
 /*****************************************************************************/
 /**
- * @brief Find command
- *
  * Find the appropriate command
  *
  * @param[in] THIS        Pointer to plugin instance data
@@ -1591,8 +1562,8 @@ static int find_command(data_t * const THIS, int streamOnly, const char * url)
 }
 
 /*****************************************************************************/
-/** NPP get Mime decription
- *
+/** 
+ * NPP get Mime decription
  * Construct a MIME Description string for netscape so that mozilla shall know
  * when to call us back.
  *
@@ -1606,7 +1577,7 @@ char *NPP_GetMIMEDescription(void)
      mimetype_t *m;
      int size_required;
 
-     D("GetMIMEDescription\n");
+     D("NPP_GetMIMEDescription()\n");
 
      do_read_config();
 
@@ -1667,8 +1638,8 @@ char *NPP_GetMIMEDescription(void)
 }
 
 /*****************************************************************************/
-/** NPP Get value
- *
+/**
+ * NPP Get value
  * Let Mozilla know things about mozplugger. This single function actually 
  * covers two interface functions NP_GetValue and NPP_GetValue. The first is
  * called without an instance pointer  (instance = future = null) and the second
@@ -1692,7 +1663,7 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
      switch (variable)
      {
      case NPPVpluginNameString:
-	  D("GET Plugin name\n");
+	  D("NP_GetValue(NPPVpluginNameString)\n");
 	  *((char **)value) = "MozPlugger "
 	       VERSION
 	       " handles QuickTime and Windows Media Player Plugin"
@@ -1700,12 +1671,12 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 	  break;
 
      case NPPVpluginDescriptionString:
-	  D("GET Plugin description\n");
+	  D("NP_GetValue(NPPVpluginDescriptionString)\n");
 	  snprintf(desc_buffer, sizeof(desc_buffer),
 		   "MozPlugger version "
 		   VERSION
-		   ", written by Fredrik H&uuml;binette, Louis Bavoil"
-                   " and Peter Leese.<br>"
+		   ", maintained by Louis Bavoil and Peter Leese, a fork of plugger"
+                   " written by Fredrik H&uuml;binette.<br>"
 		   "For documentation on how to configure mozplugger, "
 		   "check the man page. (type <tt>man&nbsp;mozplugger</tt>)"
 		   " <table> "
@@ -1733,12 +1704,49 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 	  break;
 
      case NPPVpluginNeedsXEmbed:
-          D("GET Plugin 'needs XEmbed', returned false\n"); 
-          *((int *)value) = 0;
+          D("NPP_GetValue(NPPVpluginNeedsXEmbed), returned true\n"); 
+          *((NPBool *)value) = 1;
           break; 
 
-     default:
-          D("GET Plugin value %d not implemented\n", variable);
+     default :
+#ifdef DEBUG 
+	  {
+               /* The following I've never seen in a NPP_GetValue call */
+               const char * varName;
+	       switch(variable)
+               {
+               case NPPVpluginWindowBool:
+                    varName = "NPPVpluginWindowBool";  break; 
+               case NPPVpluginTransparentBool:
+                    varName = "NPPVpluginTransparentBool"; break; 
+	       case NPPVjavaClass: 
+	            varName = "NPPVjavaClass"; break;
+               case NPPVpluginWindowSize:        
+	            varName = "NPPVpluginWindowSize"; break;
+               case NPPVpluginTimerInterval:
+	            varName = "NPPVpluginTimerInterval"; break;
+               case NPPVpluginScriptableInstance:
+	            varName = "NPPVpluginScriptableInstance"; break;
+               case NPPVpluginScriptableIID:
+	            varName = "NPPVpluginScriptableIID"; break;
+               case NPPVjavascriptPushCallerBool:
+	            varName = "NPPVjavascriptPushCallerBool"; break;
+               case NPPVpluginKeepLibraryInMemory:
+	            varName = "NPPVpluginKeepLibraryInMemory"; break;
+               case NPPVpluginScriptableNPObject:
+	            varName = "NPPVpluginScriptableNPObject"; break;
+               case NPPVformValue:
+	            varName = "NPPVformValue"; break;
+               case NPPVpluginUrlRequestsDisplayedBool:
+	            varName = "NPPVpluginUrlRequestsDisplayedBool"; break;
+               case NPPVpluginWantsAllNetworkStreams:
+	            varName = "NPPVpluginWantsNetworkStreams"; break;
+	       default:
+		    varName = ""; break;
+	       }
+	       D("NPP_GetValue('%s' - %d) not implemented\n", varName, variable);
+          }
+#endif
 	  err = NPERR_GENERIC_ERROR;
           break;
      }
@@ -1746,8 +1754,8 @@ NPError NPP_GetValue(void *instance, NPPVariable variable, void *value)
 }
 
 /*****************************************************************************/
-/** NPP Set value
- *
+/** 
+ * NPP Set value
  * Let Mozilla set things on mozplugger. 
  *
  * @param[in] instance Pointer to plugin instance data
@@ -1764,7 +1772,7 @@ NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value)
      switch (variable)
      {
      default:
-          D("SET Plugin value %d not implemented\n", variable);
+          D("NPP_SetValue( %d)  not implemented\n", variable);
 	  err = NPERR_GENERIC_ERROR;
           break;
      }
@@ -1774,8 +1782,6 @@ NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value)
 
 /*****************************************************************************/
 /**
- * @brief Special version of atoi
- *
  * Convert a string to an integer.
  * The string can be true, false, yes or no.
  *
@@ -1803,8 +1809,7 @@ static int my_atoi(const char *s, int my_true, int my_false)
 
 /*****************************************************************************/
 /**
- * @brief NPP New
- *
+ * NPP New
  * Initialize another instance of mozplugger. It is important to know
  * that there might be several instances going at one time.
  *
@@ -1840,7 +1845,7 @@ NPError NPP_New(NPMIMEType pluginType,
 
      char *url = NULL;
 
-     D("NEW (%s) - instance=%p\n", pluginType, instance);
+     D("NPP_New(%s) - instance=%p\n", pluginType, instance);
 
      if (!instance)
      {
@@ -2047,8 +2052,7 @@ NPError NPP_New(NPMIMEType pluginType,
 
 /*****************************************************************************/
 /**
- * @brief NPP Destroy
- *
+ * NPP Destroy
  * Free data, kill processes, it is time for this instance to die.
  *
  * @param[in] instance Pointer to the plugin instance data
@@ -2062,7 +2066,7 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
      int e;
      data_t * THIS;
 
-     D("Destroy() - instance=%p\n", instance);
+     D("NPP_Destroy() - instance=%p\n", instance);
 
      if (!instance)
      {
@@ -2107,8 +2111,6 @@ NPError NPP_Destroy(NPP instance, NPSavedData** save)
 
 /*****************************************************************************/
 /**
- * @brief Create new child
- *
  * Check that no child is already running before forking one.
  *
  * @param[in] THIS     Pointer to the plugin instance data
@@ -2170,8 +2172,7 @@ static void new_child(NPP instance, const char* fname)
 
 /*****************************************************************************/
 /**
- * @brief NPP New stream
- *
+ * NPP New stream
  * Open a new stream.
  * Each instance can only handle one stream at a time.
  *
@@ -2193,7 +2194,7 @@ NPError NPP_NewStream(NPP instance,
      char * savedMimetype = NULL;
      data_t * THIS;
 
-     D("NewStream() - instance=%p\n", instance);
+     D("NPP_NewStream() - instance=%p\n", instance);
 
      if (instance == NULL)
      {
@@ -2292,27 +2293,26 @@ NPError NPP_NewStream(NPP instance,
 
 /*****************************************************************************/
 /**
- * @brief NPP stream as file
- *
+ * NPP stream as file
  * Called after NPP_NewStream if *stype = NP_ASFILEONLY.
  *
  * @param[in] instance Pointer to plugin instance data
  * @param[in] stream   Pointer to the stream data structure
  * @param[in] fname    Name of the file to stream
  *
+ * @return none
+ *
  *****************************************************************************/
 void NPP_StreamAsFile(NPP instance,
 		      NPStream *stream,
 		      const char* fname)
 {
-     D("StreamAsFile() - instance=%p\n", instance);
+     D("NPP_StreamAsFile() - instance=%p\n", instance);
      new_child(instance, fname);
 }
 
 /*****************************************************************************/
 /**
- * @brief Resize the window
- *
  * The browser should have resized the window for us, but this function was
  * added because of a bug in Mozilla 1.7 (see Mozdev bug #7734) and
  * https://bugzilla.mozilla.org/show_bug.cgi?id=201158
@@ -2323,6 +2323,8 @@ void NPP_StreamAsFile(NPP instance,
  *
  * @param[in] instance Pointer to plugin instance data
  *
+ * @return none
+ *
  ****************************************************************************/
 static void resize_window(NPP instance)
 {
@@ -2351,8 +2353,7 @@ static void resize_window(NPP instance)
 
 /*****************************************************************************/
 /**
- * @brief NPP Set window
- *
+ * NPP Set window
  * The browser calls NPP_SetWindow after creating the instance to allow drawing
  * to begin. Subsequent calls to NPP_SetWindow indicate changes in size or 
  * position. If the window handle is set to null, the window is destroyed. In
@@ -2368,7 +2369,7 @@ static void resize_window(NPP instance)
 NPError NPP_SetWindow(NPP instance, NPWindow* window)
 {
      data_t * THIS;
-     D("SetWindow() - instance=%p\n", instance);
+     D("NPP_SetWindow() - instance=%p\n", instance);
 
      if (!instance)
      {
@@ -2446,8 +2447,7 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
 
 /*****************************************************************************/
 /**
- * @brief NPP Destroy stream
- *
+ * NPP Destroy stream
  * Called from the Browser when the streaming has been completed  by the Browser
  * (the reason code indicates whether this was due to a User action, Network
  * issue or that streaming has been done.
@@ -2461,14 +2461,13 @@ NPError NPP_SetWindow(NPP instance, NPWindow* window)
  *****************************************************************************/
 NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPError reason)
 {
-     D("DestroyStream() - instance=%p\n", instance);
+     D("NPP_DestroyStream() - instance=%p\n", instance);
      return NPERR_NO_ERROR;
 }
 
 /*****************************************************************************/
 /**
- * @brief NPP Initialize
- *
+ * NPP Initialize
  * The browser calls this function only once; when the plug-in is loaded, 
  * before the first instance is created. NPP_Initialize tells the plug-in that
  * the browser has loaded it.
@@ -2484,7 +2483,7 @@ NPError NPP_Initialize(void)
      NPN_Version(&pluginApiMajorVer,  &pluginApiMinorVer,
                  &browserApiMajorVer, &browserApiMinorVer);
 
-     D("Initialize - API versions plugin=%d.%d Browser=%d.%d\n", 
+     D("NPP_Initialize() - API versions plugin=%d.%d Browser=%d.%d\n", 
              pluginApiMajorVer, pluginApiMinorVer,
              browserApiMajorVer, browserApiMinorVer);
 
@@ -2502,38 +2501,39 @@ NPError NPP_Initialize(void)
 
 /*****************************************************************************/
 /**
- * @brief NPP Shutdown
- *
+ * NPP Shutdown
  * The browser calls this function just before it unloads the plugin from
  * memory. So this function should do any tidy up - in this case nothing is
  * required.
  *
+ * @return none
+ *
  *****************************************************************************/
 void NPP_Shutdown(void)
 {
-     D("Shutdown\n");
+     D("NPP_Shutdown()\n");
 }
 
 /*****************************************************************************/
 /**
- * @brief NPP Print
- *
+ * NPP Print
  * Called when user as requested to print the webpage that contains a visible
  * plug-in. For mozplugger this is ignored.
  *
  * @param[in] instance  Pointer to the plugin instance data
  * @param[in] printInfo Pointer to the print info data structure
  *
+ * @return none
+ *
  *****************************************************************************/
 void NPP_Print(NPP instance, NPPrint* printInfo)
 {
-     D("Print() - instance=%p\n", instance);
+     D("NPP_Print() - instance=%p\n", instance);
 }
 
 /*****************************************************************************/
 /**
- * @brief NPP write
- *
+ * NPP write
  * Called when the Browser wishes to deliver a block of data from a stream to
  * the plugin. Since streaming is handled directly by the application specificed
  * in the configuration file, mozplugger has no need for this data. Here it
@@ -2556,14 +2556,13 @@ int32_t NPP_Write(NPP instance,
 		int32_t len,
 		void *buf)
 {
-     D("Write(%d) - instance=%p\n", (int) len, instance);
+     D("NPP_Write(%d) - instance=%p\n", (int) len, instance);
      return len;
 }
 
 /*****************************************************************************/
 /**
- * @brief NPP Write ready
- *
+ * NPP Write ready
  * Browser calls this function before calling NPP_Write to see how much data
  * the plugin is ready to accept. Since streaming is handled directly by the
  * application specified in the configuration file, mozplugger does not need
@@ -2578,7 +2577,7 @@ int32_t NPP_Write(NPP instance,
  *****************************************************************************/
 int32_t NPP_WriteReady(NPP instance, NPStream *stream)
 {
-     D("WriteReady() - instance=%p\n", instance);
+     D("NPP_WriteReady() - instance=%p\n", instance);
      if (instance)
      {
 	  /* Tell the browser that it can finish with the stream
@@ -2591,8 +2590,7 @@ int32_t NPP_WriteReady(NPP instance, NPStream *stream)
 
 /*****************************************************************************/
 /**
- * @brief NPP URL Notify
- *
+ * NPP URL Notify
  * Browser calls this function to notify when a GET or POST has completed
  * Currently not used by mozplugger
  *
@@ -2601,8 +2599,10 @@ int32_t NPP_WriteReady(NPP instance, NPStream *stream)
  * @param[in] reason     The reason for the notify event
  * @param[in] notifyData Data that was passed in the original call to Get or Post URL
  *
+ * @return none
+ *
  *****************************************************************************/
 void NPP_URLNotify(NPP instance, const char * url, NPReason reason, void * notifyData)
 {
-     D("URLNotify() - instance=%p url=%s reason=%i\n", instance, url, reason);
+     D("NPP_URLNotify() - instance=%p url=%s reason=%i\n", instance, url, reason);
 }
