@@ -40,11 +40,6 @@
 /* Time to wait for a process to exit */
 #define KILL_TIMEOUT_USEC 100000
 
-/* Each handler has a list of mime types, and a list of commands. */
-#define MAX_NUM_HANDLERS 64
-#define MAX_NUM_TYPES_PER_HANDLER 32
-#define MAX_NUM_CMDS_PER_HANDLER 32
-
 #define MAX_STATIC_MEMORY_POOL   65536
 
 /* Maximum size of the buffer used for environment variables. */
