@@ -75,8 +75,7 @@ static NPNetscapeFuncs   gNetscapeFuncs;    /* Netscape Function table */
  *
  ***********************************************************************/
 
-void
-NPN_Version(int* plugin_major, int* plugin_minor,
+void NPN_Version(int* plugin_major, int* plugin_minor,
             int* netscape_major, int* netscape_minor)
 {
     *plugin_major = NP_VERSION_MAJOR;
@@ -89,11 +88,10 @@ NPN_Version(int* plugin_major, int* plugin_minor,
 }
 
 /******************************************************************************/
-NPError
-NPN_GetValue(NPP instance, NPNVariable variable, void *r_value)
+NPError NPN_GetValue(NPP instance, NPNVariable variable, void *r_value)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_GetValueUPP func = gNetscapeFuncs.getvalue;
+    NPN_GetValueProcPtr func = gNetscapeFuncs.getvalue;
     if(func)
     {
         retVal = (*func)(instance, variable, r_value);
@@ -102,11 +100,10 @@ NPN_GetValue(NPP instance, NPNVariable variable, void *r_value)
 }
 
 /******************************************************************************/
-NPError
-NPN_SetValue(NPP instance, NPPVariable variable, void *value)
+NPError NPN_SetValue(NPP instance, NPPVariable variable, void *value)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_SetValueUPP func = gNetscapeFuncs.setvalue;
+    NPN_SetValueProcPtr func = gNetscapeFuncs.setvalue;
     if(func)
     {
         retVal = (*func)(instance, variable, value);
@@ -115,11 +112,10 @@ NPN_SetValue(NPP instance, NPPVariable variable, void *value)
 }
 
 /******************************************************************************/
-NPError
-NPN_GetURL(NPP instance, const char* url, const char* window)
+NPError NPN_GetURL(NPP instance, const char* url, const char* window)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_GetURLUPP func = gNetscapeFuncs.geturl;
+    NPN_GetURLProcPtr func = gNetscapeFuncs.geturl;
     if(func)
     {
         retVal = (*func)(instance, url, window);
@@ -128,11 +124,11 @@ NPN_GetURL(NPP instance, const char* url, const char* window)
 }
 
 /******************************************************************************/
-NPError
-NPN_GetURLNotify(NPP instance, const char* url, const char* window, void* notifyData)
+NPError NPN_GetURLNotify(NPP instance, const char* url, const char* window, 
+                                                               void* notifyData)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_GetURLNotifyUPP func = gNetscapeFuncs.geturlnotify;
+    NPN_GetURLNotifyProcPtr func = gNetscapeFuncs.geturlnotify;
     if(func)
     {
         retVal = (*func)(instance, url, window, notifyData);
@@ -141,12 +137,11 @@ NPN_GetURLNotify(NPP instance, const char* url, const char* window, void* notify
 }
 
 /******************************************************************************/
-NPError
-NPN_PostURL(NPP instance, const char* url, const char* window,
-         uint32_t len, const char* buf, NPBool file)
+NPError NPN_PostURL(NPP instance, const char* url, const char* window,
+                                     uint32_t len, const char* buf, NPBool file)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_PostURLUPP func = gNetscapeFuncs.posturl;
+    NPN_PostURLProcPtr func = gNetscapeFuncs.posturl;
     if(func)
     {
         retVal = (*func)(instance, url, window, len, buf, file);
@@ -155,12 +150,11 @@ NPN_PostURL(NPP instance, const char* url, const char* window,
 }
 
 /******************************************************************************/
-NPError
-NPN_PostURLNotify(NPP instance, const char* url, const char* window, uint32_t len,
-                  const char* buf, NPBool file, void* notifyData)
+NPError NPN_PostURLNotify(NPP instance, const char* url, const char* window, 
+                   uint32_t len, const char* buf, NPBool file, void* notifyData)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_PostURLNotifyUPP func = gNetscapeFuncs.posturlnotify;
+    NPN_PostURLNotifyProcPtr func = gNetscapeFuncs.posturlnotify;
     if(func)
     {
         retVal = (*func)(instance, url, window, len, buf, file, notifyData);
@@ -169,11 +163,10 @@ NPN_PostURLNotify(NPP instance, const char* url, const char* window, uint32_t le
 }
 
 /******************************************************************************/
-NPError
-NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
+NPError NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_RequestReadUPP func = gNetscapeFuncs.requestread;
+    NPN_RequestReadProcPtr func = gNetscapeFuncs.requestread;
     if(func)
     {
         retVal = (*func)(stream, rangeList);
@@ -182,12 +175,11 @@ NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
 }
 
 /******************************************************************************/
-NPError
-NPN_NewStream(NPP instance, NPMIMEType type, const char *window,
-          NPStream** stream_ptr)
+NPError NPN_NewStream(NPP instance, NPMIMEType type, const char *window,
+                                                          NPStream** stream_ptr)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_NewStreamUPP func = gNetscapeFuncs.newstream;
+    NPN_NewStreamProcPtr func = gNetscapeFuncs.newstream;
     if(func)
     {
         retVal = (*func)(instance,  type, window, stream_ptr);
@@ -196,11 +188,10 @@ NPN_NewStream(NPP instance, NPMIMEType type, const char *window,
 }
 
 /******************************************************************************/
-int32_t
-NPN_Write(NPP instance, NPStream* stream, int32_t len, void* buffer)
+int32_t NPN_Write(NPP instance, NPStream* stream, int32_t len, void* buffer)
 {
     int32_t retVal = 0;
-    NPN_WriteUPP func = gNetscapeFuncs.write;
+    NPN_WriteProcPtr func = gNetscapeFuncs.write;
     if(func)
     {
         retVal = (*func)(instance, stream, len, buffer);
@@ -209,11 +200,10 @@ NPN_Write(NPP instance, NPStream* stream, int32_t len, void* buffer)
 }
 
 /******************************************************************************/
-NPError
-NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason)
+NPError NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason)
 {
     NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
-    NPN_DestroyStreamUPP func = gNetscapeFuncs.destroystream;
+    NPN_DestroyStreamProcPtr func = gNetscapeFuncs.destroystream;
     if(func)
     {
         retVal = (*func)(instance, stream, reason);
@@ -222,10 +212,9 @@ NPN_DestroyStream(NPP instance, NPStream* stream, NPError reason)
 }
 
 /******************************************************************************/
-void
-NPN_Status(NPP instance, const char* message)
+void NPN_Status(NPP instance, const char* message)
 {
-    NPN_StatusUPP func = gNetscapeFuncs.status;
+    NPN_StatusProcPtr func = gNetscapeFuncs.status;
     if(func)
     {
         (*func)(instance, message);
@@ -233,11 +222,10 @@ NPN_Status(NPP instance, const char* message)
 }
 
 /******************************************************************************/
-const char*
-NPN_UserAgent(NPP instance)
+const char* NPN_UserAgent(NPP instance)
 {
     const char * retVal = 0;
-    NPN_UserAgentUPP func = gNetscapeFuncs.uagent;
+    NPN_UserAgentProcPtr func = gNetscapeFuncs.uagent;
     if(func)
     {
         retVal = (*func)(instance);
@@ -246,11 +234,10 @@ NPN_UserAgent(NPP instance)
 }
 
 /******************************************************************************/
-void*
-NPN_MemAlloc(uint32_t size)
+void* NPN_MemAlloc(uint32_t size)
 {
     void * retVal = 0;
-    NPN_MemAllocUPP func = gNetscapeFuncs.memalloc;
+    NPN_MemAllocProcPtr func = gNetscapeFuncs.memalloc;
     if(func)
     {
         retVal = (*func)(size);
@@ -261,7 +248,7 @@ NPN_MemAlloc(uint32_t size)
 /******************************************************************************/
 void NPN_MemFree(void* ptr)
 {
-    NPN_MemFreeUPP func = gNetscapeFuncs.memfree;
+    NPN_MemFreeProcPtr func = gNetscapeFuncs.memfree;
     if(func)
     {
         (*func)(ptr);
@@ -272,7 +259,7 @@ void NPN_MemFree(void* ptr)
 uint32_t NPN_MemFlush(uint32_t size)
 {
     uint32_t retVal = 0;
-    NPN_MemFlushUPP func = gNetscapeFuncs.memflush;
+    NPN_MemFlushProcPtr func = gNetscapeFuncs.memflush;
     if(func)
     {
         retVal = (*func)(size);
@@ -283,7 +270,7 @@ uint32_t NPN_MemFlush(uint32_t size)
 /******************************************************************************/
 void NPN_ReloadPlugins(NPBool reloadPages)
 {
-    NPN_ReloadPluginsUPP func = gNetscapeFuncs.reloadplugins;
+    NPN_ReloadPluginsProcPtr func = gNetscapeFuncs.reloadplugins;
     if(func)
     {
         (*func)(reloadPages);
@@ -294,7 +281,7 @@ void NPN_ReloadPlugins(NPBool reloadPages)
 void
 NPN_InvalidateRect(NPP instance, NPRect *invalidRect)
 {
-    NPN_InvalidateRectUPP func = gNetscapeFuncs.invalidaterect;
+    NPN_InvalidateRectProcPtr func = gNetscapeFuncs.invalidaterect;
     if(func)
     {
         (*func)(instance, invalidRect);
@@ -305,7 +292,7 @@ NPN_InvalidateRect(NPP instance, NPRect *invalidRect)
 void
 NPN_InvalidateRegion(NPP instance, NPRegion invalidRegion)
 {
-    NPN_InvalidateRegionUPP func = gNetscapeFuncs.invalidateregion;
+    NPN_InvalidateRegionProcPtr func = gNetscapeFuncs.invalidateregion;
     if(func)
     {
         (*func)(instance, invalidRegion);
@@ -316,7 +303,7 @@ NPN_InvalidateRegion(NPP instance, NPRegion invalidRegion)
 void
 NPN_ForceRedraw(NPP instance)
 {
-    NPN_ForceRedrawUPP func = gNetscapeFuncs.forceredraw;
+    NPN_ForceRedrawProcPtr func = gNetscapeFuncs.forceredraw;
     if(func)
     {
         (*func)(instance);
@@ -326,7 +313,8 @@ NPN_ForceRedraw(NPP instance)
 /******************************************************************************/
 void NPN_PushPopupsEnabledState(NPP instance, NPBool enabled)
 {
-    NPN_PushPopupsEnabledStateUPP func = gNetscapeFuncs.pushpopupsenabledstate;
+    NPN_PushPopupsEnabledStateProcPtr func 
+                                       = gNetscapeFuncs.pushpopupsenabledstate;
     if(func)
     {
         (*func)(instance, enabled);
@@ -336,7 +324,8 @@ void NPN_PushPopupsEnabledState(NPP instance, NPBool enabled)
 /******************************************************************************/
 void NPN_PopPopupsEnabledState(NPP instance)
 {
-    NPN_PopPopupsEnabledStateUPP func = gNetscapeFuncs.poppopupsenabledstate;
+    NPN_PopPopupsEnabledStateProcPtr func 
+                                         = gNetscapeFuncs.poppopupsenabledstate;
     if(func)
     {
         (*func)(instance);
@@ -347,7 +336,7 @@ void NPN_PopPopupsEnabledState(NPP instance)
 NPIdentifier NPN_GetStringIdentifier(const NPUTF8 *name)
 {
     NPIdentifier retVal = 0;
-    NPN_GetStringIdentifierUPP func = gNetscapeFuncs.getstringidentifier;
+    NPN_GetStringIdentifierProcPtr func = gNetscapeFuncs.getstringidentifier;
     if(func)
     {
         retVal = (*func)(name);
@@ -357,9 +346,9 @@ NPIdentifier NPN_GetStringIdentifier(const NPUTF8 *name)
 
 /******************************************************************************/
 void NPN_GetStringIdentifiers(const NPUTF8 **names, int32_t nameCount,
-                              NPIdentifier *identifiers)
+                                                      NPIdentifier *identifiers)
 {
-    NPN_GetStringIdentifiersUPP func = gNetscapeFuncs.getstringidentifiers;
+    NPN_GetStringIdentifiersProcPtr func = gNetscapeFuncs.getstringidentifiers;
     if(func)
     {
         (*func)(names, nameCount, identifiers);
@@ -370,7 +359,7 @@ void NPN_GetStringIdentifiers(const NPUTF8 **names, int32_t nameCount,
 NPIdentifier NPN_GetIntIdentifier(int32_t intid)
 {
     NPIdentifier retVal = 0;
-    NPN_GetIntIdentifierUPP func = gNetscapeFuncs.getintidentifier;
+    NPN_GetIntIdentifierProcPtr func = gNetscapeFuncs.getintidentifier;
     if(func)
     {
         retVal = (*func)(intid);
@@ -382,7 +371,7 @@ NPIdentifier NPN_GetIntIdentifier(int32_t intid)
 bool NPN_IdentifierIsString(NPIdentifier identifier)
 {
     bool retVal = false;
-    NPN_IdentifierIsStringUPP func = gNetscapeFuncs.identifierisstring;
+    NPN_IdentifierIsStringProcPtr func = gNetscapeFuncs.identifierisstring;
     if(func)
     {
         retVal = (*func)(identifier);
@@ -394,7 +383,7 @@ bool NPN_IdentifierIsString(NPIdentifier identifier)
 NPUTF8 *NPN_UTF8FromIdentifier(NPIdentifier identifier)
 {
     NPUTF8 * retVal = 0;
-    NPN_UTF8FromIdentifierUPP func = gNetscapeFuncs.utf8fromidentifier;
+    NPN_UTF8FromIdentifierProcPtr func = gNetscapeFuncs.utf8fromidentifier;
     if(func)
     {
         retVal = (*func)(identifier);
@@ -406,7 +395,7 @@ NPUTF8 *NPN_UTF8FromIdentifier(NPIdentifier identifier)
 int32_t NPN_IntFromIdentifier(NPIdentifier identifier)
 {
     int32_t retVal = 0;
-    NPN_IntFromIdentifierUPP func = gNetscapeFuncs.intfromidentifier;
+    NPN_IntFromIdentifierProcPtr func = gNetscapeFuncs.intfromidentifier;
     if(func)
     {
         retVal = (*func)(identifier);
@@ -418,7 +407,7 @@ int32_t NPN_IntFromIdentifier(NPIdentifier identifier)
 NPObject *NPN_CreateObject(NPP npp, NPClass *aClass)
 {
     NPObject * retVal = 0;
-    NPN_CreateObjectUPP func = gNetscapeFuncs.createobject;
+    NPN_CreateObjectProcPtr func = gNetscapeFuncs.createobject;
     if(func)
     {
         retVal = (*func)( npp, aClass);
@@ -430,7 +419,7 @@ NPObject *NPN_CreateObject(NPP npp, NPClass *aClass)
 NPObject *NPN_RetainObject(NPObject *obj)
 {
     NPObject * retVal = 0;
-    NPN_RetainObjectUPP func = gNetscapeFuncs.retainobject;
+    NPN_RetainObjectProcPtr func = gNetscapeFuncs.retainobject;
     if(func)
     {
         retVal = (*func)( obj);
@@ -441,7 +430,7 @@ NPObject *NPN_RetainObject(NPObject *obj)
 /******************************************************************************/
 void NPN_ReleaseObject(NPObject *obj)
 {
-    NPN_ReleaseObjectUPP func = gNetscapeFuncs.releaseobject;
+    NPN_ReleaseObjectProcPtr func = gNetscapeFuncs.releaseobject;
     if(func)
     {
         (*func)(obj);
@@ -450,10 +439,10 @@ void NPN_ReleaseObject(NPObject *obj)
 
 /******************************************************************************/
 bool NPN_Invoke(NPP npp, NPObject* obj, NPIdentifier methodName,
-                const NPVariant *args, uint32_t argCount, NPVariant *result)
+                    const NPVariant *args, uint32_t argCount, NPVariant *result)
 {
     bool retVal = false;
-    NPN_InvokeUPP func = gNetscapeFuncs.invoke;
+    NPN_InvokeProcPtr func = gNetscapeFuncs.invoke;
     if(func)
     {
         retVal = (*func)(npp, obj, methodName,  args, argCount, result);
@@ -463,10 +452,10 @@ bool NPN_Invoke(NPP npp, NPObject* obj, NPIdentifier methodName,
 
 /******************************************************************************/
 bool NPN_InvokeDefault(NPP npp, NPObject* obj, const NPVariant *args,
-                       uint32_t argCount, NPVariant *result)
+                                           uint32_t argCount, NPVariant *result)
 {
     bool retVal = false;
-    NPN_InvokeDefaultUPP func = gNetscapeFuncs.invokeDefault;
+    NPN_InvokeDefaultProcPtr func = gNetscapeFuncs.invokeDefault;
     if(func)
     {
         retVal = (*func)(npp, obj,  args, argCount, result);
@@ -475,11 +464,10 @@ bool NPN_InvokeDefault(NPP npp, NPObject* obj, const NPVariant *args,
 }
 
 /******************************************************************************/
-bool NPN_Evaluate(NPP npp, NPObject* obj, NPString *script,
-                  NPVariant *result)
+bool NPN_Evaluate(NPP npp, NPObject* obj, NPString *script, NPVariant *result)
 {
     bool retVal = false;
-    NPN_EvaluateUPP func = gNetscapeFuncs.evaluate;
+    NPN_EvaluateProcPtr func = gNetscapeFuncs.evaluate;
     if(func)
     {
         retVal = (*func)(npp, obj, script, result);
@@ -489,10 +477,10 @@ bool NPN_Evaluate(NPP npp, NPObject* obj, NPString *script,
 
 /******************************************************************************/
 bool NPN_GetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
-                     NPVariant *result)
+                                                              NPVariant *result)
 {
     bool retVal = false;
-    NPN_GetPropertyUPP func = gNetscapeFuncs.getproperty;
+    NPN_GetPropertyProcPtr func = gNetscapeFuncs.getproperty;
     if(func)
     {
        retVal = (*func)(npp, obj, propertyName, result);
@@ -502,10 +490,10 @@ bool NPN_GetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
 
 /******************************************************************************/
 bool NPN_SetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
-                     const NPVariant *value)
+                                                         const NPVariant *value)
 {
     bool retVal = false;
-    NPN_SetPropertyUPP func = gNetscapeFuncs.setproperty;
+    NPN_SetPropertyProcPtr func = gNetscapeFuncs.setproperty;
     if(func)
     {
         retVal = (*func)(npp, obj, propertyName, value);
@@ -517,7 +505,7 @@ bool NPN_SetProperty(NPP npp, NPObject* obj, NPIdentifier propertyName,
 bool NPN_RemoveProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
 {
     bool retVal = false;
-    NPN_RemovePropertyUPP func = gNetscapeFuncs.removeproperty;
+    NPN_RemovePropertyProcPtr func = gNetscapeFuncs.removeproperty;
     if(func)
     {
         retVal = (*func)(npp, obj, propertyName);
@@ -529,7 +517,7 @@ bool NPN_RemoveProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
 bool NPN_HasProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
 {
     bool retVal = false;
-    NPN_HasPropertyUPP func = gNetscapeFuncs.hasproperty;
+    NPN_HasPropertyProcPtr func = gNetscapeFuncs.hasproperty;
     if(func)
     {
         retVal = (*func)(npp, obj, propertyName);
@@ -541,7 +529,7 @@ bool NPN_HasProperty(NPP npp, NPObject* obj, NPIdentifier propertyName)
 bool NPN_HasMethod(NPP npp, NPObject* obj, NPIdentifier methodName)
 {
     bool retVal = false;
-    NPN_HasMethodUPP func = gNetscapeFuncs.hasmethod;
+    NPN_HasMethodProcPtr func = gNetscapeFuncs.hasmethod;
     if(func)
     {
         retVal = (*func)(npp, obj, methodName);
@@ -552,7 +540,7 @@ bool NPN_HasMethod(NPP npp, NPObject* obj, NPIdentifier methodName)
 /******************************************************************************/
 void NPN_ReleaseVariantValue(NPVariant *variant)
 {
-    NPN_ReleaseVariantValueUPP func = gNetscapeFuncs.releasevariantvalue;
+    NPN_ReleaseVariantValueProcPtr func = gNetscapeFuncs.releasevariantvalue;
     if(func)
     {
         (*func)(variant);
@@ -562,19 +550,163 @@ void NPN_ReleaseVariantValue(NPVariant *variant)
 /******************************************************************************/
 void NPN_SetException(NPObject* obj, const NPUTF8 *message)
 {
-    NPN_SetExceptionUPP func = gNetscapeFuncs.setexception;
+    NPN_SetExceptionProcPtr func = gNetscapeFuncs.setexception;
     if(func)
     {
         (*func)(obj, message);
     }
 }
 
+/******************************************************************************/
+bool NPN_Enumerate(NPP npp, NPObject *obj, NPIdentifier **identifier, 
+                                                                uint32_t *count)
+{
+    NPN_EnumerateProcPtr func = gNetscapeFuncs.enumerate;
+    bool retVal = false;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, identifier, count);
+    }
+    return retVal;
+}
+
+/******************************************************************************/
+void NPN_PluginThreadAsyncCall(NPP instance, void (*func)(void *), 
+                                                                 void *userData)
+{
+    NPN_PluginThreadAsyncCallProcPtr func2 
+                                         = gNetscapeFuncs.pluginthreadasynccall;
+    if(func2)
+    {
+        (*func2)(instance, func, userData);
+    }
+}
+
+/******************************************************************************/
+bool NPN_Construct(NPP npp, NPObject* obj, const NPVariant *args, 
+                                           uint32_t argCount, NPVariant *result)
+{
+    NPN_ConstructProcPtr func = gNetscapeFuncs.construct;
+    bool retVal= false;
+    if(func)
+    {
+        retVal = (*func)(npp, obj, args, argCount, result);
+    }
+    return retVal;
+}
+
+/******************************************************************************/
+NPError NPN_GetValueForURL(NPP npp, NPNURLVariable variable, const char *url, 
+                                                    char **value, uint32_t *len)
+{
+    NPN_GetValueForURLPtr func = gNetscapeFuncs.getvalueforurl;
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    if(func)
+    {
+        retVal = (*func)(npp, variable, url, value, len);
+    }
+    return retVal;
+}
+
+/******************************************************************************/
+NPError NPN_SetValueForURL(NPP npp, NPNURLVariable variable, const char *url, 
+                                                const char *value, uint32_t len)
+{
+    NPN_SetValueForURLPtr func = gNetscapeFuncs.setvalueforurl;
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    if(func)
+    {
+        retVal = (*func)(npp, variable, url, value, len);
+    }
+    return retVal;
+}
+
+/******************************************************************************/
+NPError NPN_GetAuthenticationInfo(NPP npp, const char *protocol, 
+                                     const char *host, int32_t port, 
+                                     const char *scheme, const char *realm, 
+                                     char **username, uint32_t *ulen, 
+                                     char **password, uint32_t *plen)
+{
+    NPN_GetAuthenticationInfoPtr func = gNetscapeFuncs.getauthenticationinfo;
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    if(func)
+    {
+        retVal = (*func)(npp, protocol, host, port, scheme, realm, username, 
+                                                          ulen, password, plen);
+    }
+    return retVal;
+}
+
+/******************************************************************************/
+uint32_t NPN_ScheduleTimer(NPP instance, uint32_t interval, NPBool repeat, 
+                                   void (*timerFunc)(NPP npp, uint32_t timerID))
+{
+    NPN_ScheduleTimerPtr func = gNetscapeFuncs.scheduletimer;
+    uint32_t retVal = 0;
+    if(func)
+    {
+        retVal = (*func)(instance, interval, repeat, timerFunc);
+    }
+    return retVal;
+}
+
+/******************************************************************************/
+void NPN_UnscheduleTimer(NPP instance, uint32_t timerID)
+{
+    NPN_UnscheduleTimerPtr func = gNetscapeFuncs.unscheduletimer;
+    if(func)
+    {
+        (*func)(instance, timerID);
+    }
+}
+
+/******************************************************************************/
+NPError NPN_PopUpContextMenu(NPP instance, NPMenu* menu)
+{
+    NPN_PopUpContextMenuPtr func = gNetscapeFuncs.popupcontextmenu;
+    NPError retVal = NPERR_INVALID_FUNCTABLE_ERROR;
+    if(func)
+    {
+        retVal = (*func)(instance, menu);
+    }
+    return retVal;
+}
+
+/******************************************************************************/
+NPBool NPN_ConvertPoint(NPP instance, double sourceX, double sourceY, 
+                                   NPCoordinateSpace sourceSpace, double *destX, 
+                                     double *destY, NPCoordinateSpace destSpace)
+{
+    NPN_ConvertPointPtr func = gNetscapeFuncs.convertpoint;
+    NPBool retVal = false;
+    if(func)
+    {
+        retVal = (*func)(instance, sourceX, sourceY, sourceSpace, destX,
+                                                              destY, destSpace);
+    }
+    return retVal;
+}
+
+
 /*********************************************************************** 
  *
  * These functions are located automagically by netscape.
  *
  ***********************************************************************/
 
+/*
+ * NP_GetPluginVersion [optional]
+ *  - The browser uses the return value to indicate to the user what version of
+ *    this plugin is installed.
+ */
+char *
+NP_GetPluginVersion(void)
+{
+    return VERSION;
+}
+
+
 /*
  * NP_GetMIMEDescription
  *  - Netscape needs to know about this symbol
@@ -769,6 +901,17 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
         }
 
 
+        /* Version 0.23 */
+        if (nsTable->size >=
+            ((char *)&nsTable->convertpoint - (char *)nsTable))
+        {
+          gNetscapeFuncs.scheduletimer = nsTable->scheduletimer;
+          gNetscapeFuncs.unscheduletimer = nsTable->unscheduletimer;
+          gNetscapeFuncs.popupcontextmenu = nsTable->popupcontextmenu;
+          gNetscapeFuncs.convertpoint = nsTable->convertpoint;
+        }
+
+
         /*
          * Set up the plugin function table that Netscape will use to
          * call us.  Netscape needs to know about our version and size
@@ -793,7 +936,7 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
         /* All this is so we keep backwards compatibility with old browsers! */ 
         if (pluginFuncs->size > ((char*)&pluginFuncs->getvalue - (char*)pluginFuncs))
         {     
-            pluginFuncs->getvalue = (NPP_GetValueUPP) NPP_GetValue;
+            pluginFuncs->getvalue = (NPP_GetValueProcPtr) NPP_GetValue;
             if (pluginFuncs->size > ((char*)&pluginFuncs->setvalue - (char*)pluginFuncs))
             {    
                 pluginFuncs->setvalue = NPP_SetValue;
